@@ -32,6 +32,7 @@ from processing.algs.gdal.AssignProjection import AssignProjection
 from processing.algs.gdal.ClipRasterByExtent import ClipRasterByExtent
 from processing.algs.gdal.ClipRasterByMask import ClipRasterByMask
 from processing.algs.gdal.gdal2tiles import gdal2tiles
+from processing.algs.gdal.gdalcalc import gdalcalc
 from processing.algs.gdal.gdaltindex import gdaltindex
 from processing.algs.gdal.contour import contour
 from processing.algs.gdal.GridAverage import GridAverage
@@ -40,11 +41,15 @@ from processing.algs.gdal.GridInverseDistance import GridInverseDistance
 from processing.algs.gdal.GridInverseDistanceNearestNeighbor import GridInverseDistanceNearestNeighbor
 from processing.algs.gdal.GridLinear import GridLinear
 from processing.algs.gdal.GridNearestNeighbor import GridNearestNeighbor
+from processing.algs.gdal.buildvrt import buildvrt
+from processing.algs.gdal.hillshade import hillshade
+from processing.algs.gdal.ogr2ogr import ogr2ogr
 from processing.algs.gdal.proximity import proximity
 from processing.algs.gdal.rasterize import rasterize
 from processing.algs.gdal.retile import retile
 from processing.algs.gdal.translate import translate
 from processing.algs.gdal.warp import warp
+from processing.algs.gdal.fillnodata import fillnodata
 
 from qgis.core import (QgsProcessingContext,
                        QgsProcessingFeedback,
@@ -55,8 +60,9 @@ from qgis.core import (QgsProcessingContext,
                        QgsGeometry,
                        QgsPointXY,
                        QgsProject,
+                       QgsVectorLayer,
                        QgsRectangle,
-                       QgsProcessingUtils,
+                       QgsProcessingException,
                        QgsProcessingFeatureSourceDefinition)
 import nose2
 import os
@@ -94,6 +100,24 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
         for a in p.algorithms():
             self.assertTrue(a.commandName(), 'Algorithm {} has no commandName!'.format(a.id()))
 
+    def testCommandNameInTags(self):
+        # Test that algorithms commandName is present in provided tags
+        p = QgsApplication.processingRegistry().providerById('gdal')
+        for a in p.algorithms():
+            self.assertTrue(a.commandName() in a.tags(), 'Algorithm {} commandName not found in tags!'.format(a.id()))
+
+    def testNoParameters(self):
+        # Test that algorithms throw QgsProcessingExceptions and not base Python
+        # exceptions when no parameters specified
+        p = QgsApplication.processingRegistry().providerById('gdal')
+        context = QgsProcessingContext()
+        feedback = QgsProcessingFeedback()
+        for a in p.algorithms():
+            try:
+                a.getConsoleCommands({}, context, feedback)
+            except QgsProcessingException:
+                pass
+
     def testGetOgrCompatibleSourceFromMemoryLayer(self):
         # create a memory layer and add to project and context
         layer = QgsVectorLayer("Point?field=fldtxt:string&field=fldint:integer",
@@ -129,8 +153,42 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
         self.assertTrue(res.isValid())
         self.assertEqual(res.featureCount(), 2)
 
+        # with memory layers - if not executing layer source should be ignored and replaced
+        # with a dummy path, because:
+        # - it has no meaning for the gdal command outside of QGIS, memory layers don't exist!
+        # - we don't want to force an export of the whole memory layer to a temp file just to show the command preview
+        # this might be very slow!
+        ogr_data_path, ogr_layer_name = alg.getOgrCompatibleSource('INPUT', parameters, context, feedback,
+                                                                   executing=False)
+        self.assertEqual(ogr_data_path, 'path_to_data_file')
+        self.assertEqual(ogr_layer_name, 'layer_name')
+
         QgsProject.instance().removeMapLayer(layer)
 
+    def testGetOgrCompatibleSourceFromOgrLayer(self):
+        p = QgsProject()
+        source = os.path.join(testDataPath, 'points.gml')
+        vl = QgsVectorLayer(source)
+        self.assertTrue(vl.isValid())
+        p.addMapLayer(vl)
+
+        context = QgsProcessingContext()
+        context.setProject(p)
+        feedback = QgsProcessingFeedback()
+
+        alg = ogr2ogr()
+        alg.initAlgorithm()
+        path, layer = alg.getOgrCompatibleSource('INPUT', {'INPUT': vl.id()}, context, feedback, True)
+        self.assertEqual(path, source)
+        path, layer = alg.getOgrCompatibleSource('INPUT', {'INPUT': vl.id()}, context, feedback, False)
+        self.assertEqual(path, source)
+
+        # with selected features only - if not executing, the 'selected features only' setting
+        # should be ignored (because it has no meaning for the gdal command outside of QGIS!)
+        parameters = {'INPUT': QgsProcessingFeatureSourceDefinition(vl.id(), True)}
+        path, layer = alg.getOgrCompatibleSource('INPUT', parameters, context, feedback, False)
+        self.assertEqual(path, source)
+
     def testGetOgrCompatibleSourceFromFeatureSource(self):
         # create a memory layer and add to project and context
         layer = QgsVectorLayer("Point?field=fldtxt:string&field=fldint:integer",
@@ -214,10 +272,10 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
     def testOgrConnectionStringAndFormat(self):
         context = QgsProcessingContext()
         output, outputFormat = GdalUtils.ogrConnectionStringAndFormat('d:/test/test.shp', context)
-        self.assertEqual(output, '"d:/test/test.shp"')
+        self.assertEqual(output, 'd:/test/test.shp')
         self.assertEqual(outputFormat, '"ESRI Shapefile"')
         output, outputFormat = GdalUtils.ogrConnectionStringAndFormat('d:/test/test.mif', context)
-        self.assertEqual(output, '"d:/test/test.mif"')
+        self.assertEqual(output, 'd:/test/test.mif')
         self.assertEqual(outputFormat, '"MapInfo File"')
 
     def testCrsConversion(self):
@@ -290,6 +348,15 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
              '-ot Float32 -of JPEG ' +
              source + ' ' +
              'd:/temp/check.jpg'])
+        # with None NODATA value
+        self.assertEqual(
+            translate_alg.getConsoleCommands({'INPUT': source,
+                                              'NODATA': None,
+                                              'OUTPUT': 'd:/temp/check.jpg'}, context, feedback),
+            ['gdal_translate',
+             '-ot Float32 -of JPEG ' +
+             source + ' ' +
+             'd:/temp/check.jpg'])
         # with NODATA value
         self.assertEqual(
             translate_alg.getConsoleCommands({'INPUT': source,
@@ -460,7 +527,7 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
             ['gdal_contour',
              '-b 1 -a elev -i 5.0 -f "ESRI Shapefile" ' +
              source + ' ' +
-             '"d:/temp/check.shp"'])
+             'd:/temp/check.shp'])
         # with NODATA value
         self.assertEqual(
             alg.getConsoleCommands({'INPUT': source,
@@ -472,7 +539,7 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
             ['gdal_contour',
              '-b 1 -a elev -i 5.0 -snodata 9999.0 -f "ESRI Shapefile" ' +
              source + ' ' +
-             '"d:/temp/check.shp"'])
+             'd:/temp/check.shp'])
         # with "0" NODATA value
         self.assertEqual(
             alg.getConsoleCommands({'INPUT': source,
@@ -484,7 +551,7 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
             ['gdal_contour',
              '-b 1 -a elev -i 5.0 -snodata 0.0 -f "GPKG" ' +
              source + ' ' +
-             '"d:/temp/check.gpkg"'])
+             'd:/temp/check.gpkg'])
 
     def testGdal2Tiles(self):
         context = QgsProcessingContext()
@@ -562,6 +629,180 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
              source + ' ' +
              'd:/temp/'])
 
+    def testGdalCalc(self):
+        context = QgsProcessingContext()
+        feedback = QgsProcessingFeedback()
+        source = os.path.join(testDataPath, 'dem.tif')
+        alg = gdalcalc()
+        alg.initAlgorithm()
+
+        output = 'd:/temp/check.jpg'
+
+        # default execution
+        formula = 'A*2' # default formula
+        self.assertEqual(
+            alg.getConsoleCommands({
+                'INPUT_A': source,
+                'BAND_A': 1,
+                'FORMULA': formula,
+                'BAND_D': -1,
+                'NO_DATA': None,
+                'BAND_F': -1,
+                'BAND_B': -1,
+                'EXTRA': '',
+                'RTYPE': 5,
+                'INPUT_F': None,
+                'BAND_E': -1,
+                'INPUT_D': None,
+                'INPUT_B': None,
+                'BAND_C': -1,
+                'INPUT_E': None,
+                'INPUT_C': None,
+                'OUTPUT': output}, context, feedback),
+            ['gdal_calc', '--calc "{}" --format JPEG --type Float32 -A {} --A_band 1 --outfile {}'.format(formula, source, output)])
+
+        # check that formula is not escaped and formula is returned as it is
+        formula = 'A * 2'  # <--- add spaces in the formula
+        self.assertEqual(
+            alg.getConsoleCommands({
+                'INPUT_A': source,
+                'BAND_A': 1,
+                'FORMULA': formula,
+                'BAND_D': -1,
+                'NO_DATA': None,
+                'BAND_F': -1,
+                'BAND_B': -1,
+                'EXTRA': '',
+                'RTYPE': 5,
+                'INPUT_F': None,
+                'BAND_E': -1,
+                'INPUT_D': None,
+                'INPUT_B': None,
+                'BAND_C': -1,
+                'INPUT_E': None,
+                'INPUT_C': None,
+                'OUTPUT': output}, context, feedback),
+            ['gdal_calc', '--calc "{}" --format JPEG --type Float32 -A {} --A_band 1 --outfile {}'.format(formula, source, output)])
+
+    def testBuildVrt(self):
+        context = QgsProcessingContext()
+        feedback = QgsProcessingFeedback()
+        source = os.path.join(testDataPath, 'dem.tif')
+        alg = buildvrt()
+        alg.initAlgorithm()
+
+        commands = alg.getConsoleCommands({'LAYERS': [source],
+                                           'OUTPUT': 'd:/temp/test.vrt'}, context, feedback)
+        self.assertEqual(len(commands), 2)
+        self.assertEqual(commands[0], 'gdalbuildvrt')
+        self.assertIn('-resolution average', commands[1])
+        self.assertIn('-separate', commands[1])
+        self.assertNotIn('-allow_projection_difference', commands[1])
+        self.assertNotIn('-add_alpha', commands[1])
+        self.assertNotIn('-a_srs', commands[1])
+        self.assertIn('-r nearest', commands[1])
+        self.assertIn('-input_file_list', commands[1])
+        self.assertIn('d:/temp/test.vrt', commands[1])
+
+        commands = alg.getConsoleCommands({'LAYERS': [source],
+                                           'RESOLUTION': 2,
+                                           'OUTPUT': 'd:/temp/test.vrt'}, context, feedback)
+        self.assertEqual(len(commands), 2)
+        self.assertEqual(commands[0], 'gdalbuildvrt')
+        self.assertIn('-resolution lowest', commands[1])
+        self.assertIn('-separate', commands[1])
+        self.assertNotIn('-allow_projection_difference', commands[1])
+        self.assertNotIn('-add_alpha', commands[1])
+        self.assertNotIn('-a_srs', commands[1])
+        self.assertIn('-r nearest', commands[1])
+        self.assertIn('-input_file_list', commands[1])
+        self.assertIn('d:/temp/test.vrt', commands[1])
+
+        commands = alg.getConsoleCommands({'LAYERS': [source],
+                                           'SEPARATE': False,
+                                           'OUTPUT': 'd:/temp/test.vrt'}, context, feedback)
+        self.assertEqual(len(commands), 2)
+        self.assertEqual(commands[0], 'gdalbuildvrt')
+        self.assertIn('-resolution average', commands[1])
+        self.assertNotIn('-allow_projection_difference', commands[1])
+        self.assertNotIn('-separate', commands[1])
+        self.assertNotIn('-add_alpha', commands[1])
+        self.assertNotIn('-a_srs', commands[1])
+        self.assertIn('-r nearest', commands[1])
+        self.assertIn('-input_file_list', commands[1])
+        self.assertIn('d:/temp/test.vrt', commands[1])
+
+        commands = alg.getConsoleCommands({'LAYERS': [source],
+                                           'PROJ_DIFFERENCE': True,
+                                           'OUTPUT': 'd:/temp/test.vrt'}, context, feedback)
+        self.assertEqual(len(commands), 2)
+        self.assertEqual(commands[0], 'gdalbuildvrt')
+        self.assertIn('-resolution average', commands[1])
+        self.assertIn('-allow_projection_difference', commands[1])
+        self.assertIn('-separate', commands[1])
+        self.assertNotIn('-add_alpha', commands[1])
+        self.assertNotIn('-a_srs', commands[1])
+        self.assertIn('-r nearest', commands[1])
+        self.assertIn('-input_file_list', commands[1])
+        self.assertIn('d:/temp/test.vrt', commands[1])
+
+        commands = alg.getConsoleCommands({'LAYERS': [source],
+                                           'ADD_ALPHA': True,
+                                           'OUTPUT': 'd:/temp/test.vrt'}, context, feedback)
+        self.assertEqual(len(commands), 2)
+        self.assertEqual(commands[0], 'gdalbuildvrt')
+        self.assertIn('-resolution average', commands[1])
+        self.assertIn('-separate', commands[1])
+        self.assertNotIn('-allow_projection_difference', commands[1])
+        self.assertNotIn('-add_alpha', commands[1])
+        self.assertNotIn('-a_srs', commands[1])
+        self.assertIn('-r nearest', commands[1])
+        self.assertIn('-input_file_list', commands[1])
+        self.assertIn('d:/temp/test.vrt', commands[1])
+
+        commands = alg.getConsoleCommands({'LAYERS': [source],
+                                           'ASSIGN_CRS': 'EPSG:3111',
+                                           'OUTPUT': 'd:/temp/test.vrt'}, context, feedback)
+        self.assertEqual(len(commands), 2)
+        self.assertEqual(commands[0], 'gdalbuildvrt')
+        self.assertIn('-resolution average', commands[1])
+        self.assertIn('-separate', commands[1])
+        self.assertNotIn('-allow_projection_difference', commands[1])
+        self.assertNotIn('-add_alpha', commands[1])
+        self.assertIn('-a_srs EPSG:3111', commands[1])
+        self.assertIn('-r nearest', commands[1])
+        self.assertIn('-input_file_list', commands[1])
+        self.assertIn('d:/temp/test.vrt', commands[1])
+
+        custom_crs = 'proj4: +proj=utm +zone=36 +south +a=6378249.145 +b=6356514.966398753 +towgs84=-143,-90,-294,0,0,0,0 +units=m +no_defs'
+        commands = alg.getConsoleCommands({'LAYERS': [source],
+                                           'ASSIGN_CRS': custom_crs,
+                                           'OUTPUT': 'd:/temp/test.vrt'}, context, feedback)
+        self.assertEqual(len(commands), 2)
+        self.assertEqual(commands[0], 'gdalbuildvrt')
+        self.assertIn('-resolution average', commands[1])
+        self.assertIn('-separate', commands[1])
+        self.assertNotIn('-allow_projection_difference', commands[1])
+        self.assertNotIn('-add_alpha', commands[1])
+        self.assertIn('-a_srs EPSG:20936', commands[1])
+        self.assertIn('-r nearest', commands[1])
+        self.assertIn('-input_file_list', commands[1])
+        self.assertIn('d:/temp/test.vrt', commands[1])
+
+        commands = alg.getConsoleCommands({'LAYERS': [source],
+                                           'RESAMPLING': 4,
+                                           'OUTPUT': 'd:/temp/test.vrt'}, context, feedback)
+        self.assertEqual(len(commands), 2)
+        self.assertEqual(commands[0], 'gdalbuildvrt')
+        self.assertIn('-resolution average', commands[1])
+        self.assertIn('-separate', commands[1])
+        self.assertNotIn('-allow_projection_difference', commands[1])
+        self.assertNotIn('-add_alpha', commands[1])
+        self.assertNotIn('-a_srs', commands[1])
+        self.assertIn('-r lanczos', commands[1])
+        self.assertIn('-input_file_list', commands[1])
+        self.assertIn('d:/temp/test.vrt', commands[1])
+
     def testGdalTindex(self):
         context = QgsProcessingContext()
         feedback = QgsProcessingFeedback()
@@ -574,7 +815,7 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
                                     'OUTPUT': 'd:/temp/test.shp'}, context, feedback),
             ['gdaltindex',
              '-tileindex location -f "ESRI Shapefile" ' +
-             '"d:/temp/test.shp" ' +
+             'd:/temp/test.shp ' +
              source])
 
         # with input srs
@@ -584,7 +825,7 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
                                     'OUTPUT': 'd:/temp/test.shp'}, context, feedback),
             ['gdaltindex',
              '-tileindex location -t_srs EPSG:3111 -f "ESRI Shapefile" ' +
-             '"d:/temp/test.shp" ' +
+             'd:/temp/test.shp ' +
              source])
 
         # with target using proj string
@@ -595,7 +836,7 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
                                     'OUTPUT': 'd:/temp/test.shp'}, context, feedback),
             ['gdaltindex',
              '-tileindex location -t_srs EPSG:20936 -f "ESRI Shapefile" ' +
-             '"d:/temp/test.shp" ' +
+             'd:/temp/test.shp ' +
              source])
 
         # with target using custom projection
@@ -606,7 +847,7 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
                                     'OUTPUT': 'd:/temp/test.shp'}, context, feedback),
             ['gdaltindex',
              '-tileindex location -t_srs "+proj=utm +zone=36 +south +a=63785 +b=6357 +towgs84=-143,-90,-294,0,0,0,0 +units=m +no_defs" -f "ESRI Shapefile" ' +
-             '"d:/temp/test.shp" ' +
+             'd:/temp/test.shp ' +
              source])
 
         # with non-EPSG crs code
@@ -616,7 +857,7 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
                                     'OUTPUT': 'd:/temp/test.shp'}, context, feedback),
             ['gdaltindex',
              '-tileindex location -t_srs EPSG:3111 -f "ESRI Shapefile" ' +
-             '"d:/temp/test.shp" ' +
+             'd:/temp/test.shp ' +
              source])
 
     def testGridAverage(self):
@@ -733,7 +974,7 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
             alg.getConsoleCommands({'INPUT': source,
                                     'OUTPUT': 'd:/temp/check.jpg'}, context, feedback),
             ['gdal_grid',
-             '-l points -a invdistnn:power=2.0:smothing=0.0:radius=1.0:max_points=0:min_points=0:nodata=0.0 -ot Float32 -of JPEG ' +
+             '-l points -a invdistnn:power=2.0:smothing=0.0:radius=1.0:max_points=12:min_points=0:nodata=0.0 -ot Float32 -of JPEG ' +
              source + ' ' +
              'd:/temp/check.jpg'])
         # with NODATA value
@@ -742,7 +983,7 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
                                     'NODATA': 9999,
                                     'OUTPUT': 'd:/temp/check.jpg'}, context, feedback),
             ['gdal_grid',
-             '-l points -a invdistnn:power=2.0:smothing=0.0:radius=1.0:max_points=0:min_points=0:nodata=9999.0 -ot Float32 -of JPEG ' +
+             '-l points -a invdistnn:power=2.0:smothing=0.0:radius=1.0:max_points=12:min_points=0:nodata=9999.0 -ot Float32 -of JPEG ' +
              source + ' ' +
              'd:/temp/check.jpg'])
         # with "0" NODATA value
@@ -751,7 +992,7 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
                                     'NODATA': 0,
                                     'OUTPUT': 'd:/temp/check.jpg'}, context, feedback),
             ['gdal_grid',
-             '-l points -a invdistnn:power=2.0:smothing=0.0:radius=1.0:max_points=0:min_points=0:nodata=0.0 -ot Float32 -of JPEG ' +
+             '-l points -a invdistnn:power=2.0:smothing=0.0:radius=1.0:max_points=12:min_points=0:nodata=0.0 -ot Float32 -of JPEG ' +
              source + ' ' +
              'd:/temp/check.jpg'])
 
@@ -823,6 +1064,129 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
              source + ' ' +
              'd:/temp/check.jpg'])
 
+    def testOgr2Ogr(self):
+        context = QgsProcessingContext()
+        feedback = QgsProcessingFeedback()
+        source = os.path.join(testDataPath, 'polys.gml')
+        alg = ogr2ogr()
+        alg.initAlgorithm()
+
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'OUTPUT': 'd:/temp/check.shp'}, context, feedback),
+            ['ogr2ogr',
+             '-f "ESRI Shapefile" d:/temp/check.shp ' +
+             source + ' polys2'])
+
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'OUTPUT': 'd:/temp/check.kml'}, context, feedback),
+            ['ogr2ogr',
+             '-f "LIBKML" d:/temp/check.kml ' +
+             source + ' polys2'])
+
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'OUTPUT': 'd:/temp/my out/check.kml'}, context, feedback),
+            ['ogr2ogr',
+             '-f "LIBKML" "d:/temp/my out/check.kml" ' +
+             source + ' polys2'])
+
+    def testHillshade(self):
+        context = QgsProcessingContext()
+        feedback = QgsProcessingFeedback()
+        source = os.path.join(testDataPath, 'dem.tif')
+        alg = hillshade()
+        alg.initAlgorithm()
+
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'BAND': 1,
+                                    'Z_FACTOR': 5,
+                                    'SCALE': 2,
+                                    'AZIMUTH': 90,
+                                    'ALTITUDE': 20,
+                                    'OUTPUT': 'd:/temp/check.tif'}, context, feedback),
+            ['gdaldem',
+             'hillshade ' +
+             source + ' ' +
+             'd:/temp/check.tif -of GTiff -b 1 -z 5.0 -s 2.0 -az 90.0 -alt 20.0'])
+
+        #paths with space
+        source_with_space = os.path.join(testDataPath, 'raster with spaces.tif')
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source_with_space,
+                                    'BAND': 1,
+                                    'Z_FACTOR': 5,
+                                    'SCALE': 2,
+                                    'AZIMUTH': 90,
+                                    'ALTITUDE': 20,
+                                    'OUTPUT': 'd:/temp/check out.tif'}, context, feedback),
+            ['gdaldem',
+             'hillshade ' +
+             '"' + source_with_space + '" ' +
+             '"d:/temp/check out.tif" -of GTiff -b 1 -z 5.0 -s 2.0 -az 90.0 -alt 20.0'])
+
+        # compute edges
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'BAND': 1,
+                                    'Z_FACTOR': 5,
+                                    'SCALE': 2,
+                                    'AZIMUTH': 90,
+                                    'ALTITUDE': 20,
+                                    'COMPUTE_EDGES': True,
+                                    'OUTPUT': 'd:/temp/check.tif'}, context, feedback),
+            ['gdaldem',
+             'hillshade ' +
+             source + ' ' +
+             'd:/temp/check.tif -of GTiff -b 1 -z 5.0 -s 2.0 -az 90.0 -alt 20.0 -compute_edges'])
+
+        # with ZEVENBERGEN
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'BAND': 1,
+                                    'Z_FACTOR': 5,
+                                    'SCALE': 2,
+                                    'AZIMUTH': 90,
+                                    'ALTITUDE': 20,
+                                    'ZEVENBERGEN': True,
+                                    'OUTPUT': 'd:/temp/check.tif'}, context, feedback),
+            ['gdaldem',
+             'hillshade ' +
+             source + ' ' +
+             'd:/temp/check.tif -of GTiff -b 1 -z 5.0 -s 2.0 -az 90.0 -alt 20.0 -alg ZevenbergenThorne'])
+
+        # with COMBINED
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'BAND': 1,
+                                    'Z_FACTOR': 5,
+                                    'SCALE': 2,
+                                    'AZIMUTH': 90,
+                                    'ALTITUDE': 20,
+                                    'COMBINED': True,
+                                    'OUTPUT': 'd:/temp/check.tif'}, context, feedback),
+            ['gdaldem',
+             'hillshade ' +
+             source + ' ' +
+             'd:/temp/check.tif -of GTiff -b 1 -z 5.0 -s 2.0 -az 90.0 -alt 20.0 -combined'])
+
+        # with multidirectional - "az" argument is not allowed!
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'BAND': 1,
+                                    'Z_FACTOR': 5,
+                                    'SCALE': 2,
+                                    'AZIMUTH': 90,
+                                    'ALTITUDE': 20,
+                                    'MULTIDIRECTIONAL': True,
+                                    'OUTPUT': 'd:/temp/check.tif'}, context, feedback),
+            ['gdaldem',
+             'hillshade ' +
+             source + ' ' +
+             'd:/temp/check.tif -of GTiff -b 1 -z 5.0 -s 2.0 -alt 20.0 -multidirectional'])
+
     def testProximity(self):
         context = QgsProcessingContext()
         feedback = QgsProcessingFeedback()
@@ -965,6 +1329,16 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
              '-s_srs EPSG:3111 -t_srs EPSG:4326 -r near -ot Float32 -of JPEG ' +
              source + ' ' +
              'd:/temp/check.jpg'])
+        # with None NODATA value
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'NODATA': None,
+                                    'SOURCE_CRS': 'EPSG:3111',
+                                    'OUTPUT': 'd:/temp/check.jpg'}, context, feedback),
+            ['gdalwarp',
+             '-s_srs EPSG:3111 -t_srs EPSG:4326 -r near -ot Float32 -of JPEG ' +
+             source + ' ' +
+             'd:/temp/check.jpg'])
         # with NODATA value
         self.assertEqual(
             alg.getConsoleCommands({'INPUT': source,
@@ -1021,6 +1395,92 @@ class TestGdalAlgorithms(unittest.TestCase, AlgorithmsTestBase.AlgorithmsTest):
              source + ' ' +
              'd:/temp/check.jpg'])
 
+        # with target resolution with None value
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'SOURCE_CRS': 'EPSG:3111',
+                                    'TARGET_RESOLUTION': None,
+                                    'OUTPUT': 'd:/temp/check.jpg'}, context, feedback),
+            ['gdalwarp',
+             '-s_srs EPSG:3111 -t_srs EPSG:4326 -r near -ot Float32 -of JPEG ' +
+             source + ' ' +
+             'd:/temp/check.jpg'])
+
+        # test target resolution with a valid value
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'SOURCE_CRS': 'EPSG:3111',
+                                    'TARGET_RESOLUTION': 10.0,
+                                    'OUTPUT': 'd:/temp/check.jpg'}, context, feedback),
+            ['gdalwarp',
+             '-s_srs EPSG:3111 -t_srs EPSG:4326 -tr 10.0 10.0 -r near -ot Float32 -of JPEG ' +
+             source + ' ' +
+             'd:/temp/check.jpg'])
+
+        # test target resolution with a value of zero, to be ignored
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'SOURCE_CRS': 'EPSG:3111',
+                                    'TARGET_RESOLUTION': 0.0,
+                                    'OUTPUT': 'd:/temp/check.jpg'}, context, feedback),
+            ['gdalwarp',
+             '-s_srs EPSG:3111 -t_srs EPSG:4326 -r near -ot Float32 -of JPEG ' +
+             source + ' ' +
+             'd:/temp/check.jpg'])
+
+    def testFillnodata(self):
+        context = QgsProcessingContext()
+        feedback = QgsProcessingFeedback()
+        source = os.path.join(testDataPath, 'dem.tif')
+        mask = os.path.join(testDataPath, 'raster.tif')
+        outsource = 'd:/temp/check.tif'
+        alg = fillnodata()
+        alg.initAlgorithm()
+
+        # with mask value
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'BAND': 1,
+                                    'DISTANCE': 10,
+                                    'ITERATIONS': 0,
+                                    'MASK_LAYER': mask,
+                                    'NO_MASK': False,
+                                    'OUTPUT': outsource}, context, feedback),
+            ['gdal_fillnodata.py',
+             '-md 10 -b 1 -mask ' +
+             mask +
+             ' -of GTiff ' +
+             source + ' ' +
+             outsource])
+
+        # without mask value
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'BAND': 1,
+                                    'DISTANCE': 10,
+                                    'ITERATIONS': 0,
+                                    'NO_MASK': False,
+                                    'OUTPUT': outsource}, context, feedback),
+            ['gdal_fillnodata.py',
+             '-md 10 -b 1 ' +
+             '-of GTiff ' +
+             source + ' ' +
+             outsource])
+
+        # nomask true
+        self.assertEqual(
+            alg.getConsoleCommands({'INPUT': source,
+                                    'BAND': 1,
+                                    'DISTANCE': 10,
+                                    'ITERATIONS': 0,
+                                    'NO_MASK': True,
+                                    'OUTPUT': outsource}, context, feedback),
+            ['gdal_fillnodata.py',
+             '-md 10 -b 1 -nomask ' +
+             '-of GTiff ' +
+             source + ' ' +
+             outsource])
+
 
 class TestGdalOgrToPostGis(unittest.TestCase):
 
